<?php
/**
 * All-in-One WP Migration 
 * Copyright (C) 2014-2018 ServMask Inc.
 *
 * Modified Copyright (C) 2025 Lead Connector
 * 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
if ( ! defined( 'ABSPATH' ) ) {
	die( 'Well Sir You Cannot Jump Here...haha' );
}

class Lcm_Archive_Exception extends Exception {}
class Lcm_Backups_Exception extends Exception {}
class Lcm_Export_Exception extends Exception {}
class Lcm_Http_Exception extends Exception {}
class Lcm_Import_Exception extends Exception {}
class Lcm_Import_Retry_Exception extends Exception {}
class Lcm_Not_Accessible_Exception extends Exception {}
class Lcm_Not_Seekable_Exception extends Exception {}
class Lcm_Not_Tellable_Exception extends Exception {}
class Lcm_Not_Readable_Exception extends Exception {}
class Lcm_Not_Writable_Exception extends Exception {}
class Lcm_Not_Truncatable_Exception extends Exception {}
class Lcm_Not_Closable_Exception extends Exception {}
class Lcm_Not_Found_Exception extends Exception {}
class Lcm_Not_Directory_Exception extends Exception {}
class Lcm_Not_Valid_Secret_Key_Exception extends Exception {}
class Lcm_Quota_Exceeded_Exception extends Exception {}
class Lcm_Storage_Exception extends Exception {}
class Lcm_Compatibility_Exception extends Exception {}
class Lcm_Feedback_Exception extends Exception {}
class Lcm_Database_Exception extends Exception {}
class Lcm_Not_Encryptable_Exception extends Exception {}
class Lcm_Not_Decryptable_Exception extends Exception {}
class Lcm_Connect_Exception extends Exception {}
class Lcm_Rate_Limit_Exception extends Exception {}
class Lcm_Internal_Server_Error_Exception extends Exception {}
class Lcm_Permission_Denied_Exception extends Exception {}
class Lcm_Error_Exception extends Exception {}
class Lcm_Upload_Retry_Exception extends Exception {}
class Lcm_Upload_Exception extends Exception {}
class Lcm_Event_Exception extends Exception {}	